@@ -3,7 +3,7 @@
 
 # The MIT License (MIT)
 
-# Copyright (c) 2014-2016 CNRS
+# Copyright (c) 2014-2019 CNRS
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -26,65 +26,148 @@
 # AUTHORS
 # Hervé BREDIN - http://herve.niderb.fr
 
-from __future__ import unicode_literals
-from __future__ import print_function
+"""
+#############
+Visualization
+#############
 
-from IPython.core.pylabtools import print_figure
-from matplotlib.cm import get_cmap
-import six.moves
-from unidecode import unidecode
-import tempfile
-import networkx as nx
+:class:`pyannote.core.Segment`, :class:`pyannote.core.Timeline`,
+:class:`pyannote.core.Annotation` and :class:`pyannote.core.SlidingWindowFeature`
+instances can be directly visualized in notebooks.
+
+You will however need to install ``pytannote.core``'s additional dependencies
+for notebook representations (namely, matplotlib):
+
+
+.. code-block:: bash
+
+    pip install pyannote.core[notebook]
+
+
+Segments
+--------
+
+.. code-block:: ipython
+
+  In [1]: from pyannote.core import Segment
+
+  In [2]: segment = Segment(start=5, end=15)
+    ....: segment
+
+.. plot:: pyplots/segment.py
+
+
+Timelines
+---------
+
+.. code-block:: ipython
+
+  In [25]: from pyannote.core import Timeline, Segment
+
+  In [26]: timeline = Timeline()
+     ....: timeline.add(Segment(1, 5))
+     ....: timeline.add(Segment(6, 8))
+     ....: timeline.add(Segment(12, 18))
+     ....: timeline.add(Segment(7, 20))
+     ....: timeline
+
+.. plot:: pyplots/timeline.py
+
+
+Annotations
+-----------
+
+
+.. code-block:: ipython
+
+    In [1]: from pyannote.core import Annotation, Segment
+
+    In [6]: annotation = Annotation()
+       ...: annotation[Segment(1, 5)] = 'Carol'
+       ...: annotation[Segment(6, 8)] = 'Bob'
+       ...: annotation[Segment(12, 18)] = 'Carol'
+       ...: annotation[Segment(7, 20)] = 'Alice'
+       ...: annotation
+
+.. plot:: pyplots/annotation.py
+
+"""
+from typing import Iterable, Dict, Optional
+
+from .utils.types import Label, LabelStyle, Resource
+
+try:
+    from IPython.core.pylabtools import print_figure
+except Exception as e:
+    pass
 import numpy as np
-import subprocess
 from itertools import cycle, product, groupby
 from .segment import Segment
 from .timeline import Timeline
 from .annotation import Annotation
-from .scores import Scores
+from .feature import SlidingWindowFeature
+
+try:
+    import matplotlib
+except ImportError:
+    MATPLOTLIB_IS_AVAILABLE = False
+else:
+    MATPLOTLIB_IS_AVAILABLE = True
+
+MATPLOTLIB_WARNING = \
+    "Couldn't import matplotlib to render the vizualization " \
+    "for object {klass}. To enable, install the required dependencies " \
+    "with 'pip install pyannore.core[notebook]'"
 
 
-class Notebook(object):
+class Notebook:
 
     def __init__(self):
-        super(Notebook, self).__init__()
         self.reset()
 
     def reset(self):
-        linewidth = [2, 4]
+        from matplotlib.cm import get_cmap
+
+        linewidth = [3, 1]
         linestyle = ['solid', 'dashed', 'dotted']
 
         cm = get_cmap('Set1')
         colors = [cm(1. * i / 8) for i in range(9)]
 
-        self._style_generator = cycle(product(linewidth, linestyle, colors))
-        self._style = {None: (1, 'solid', (0.0, 0.0, 0.0))}
+        self._style_generator = cycle(product(linestyle, linewidth, colors))
+        self._style: Dict[Optional[Label], LabelStyle] = {
+            None: ('solid', 1, (0.0, 0.0, 0.0))
+        }
         del self.crop
         del self.width
 
-    def crop():
-        doc = "The crop property."
-        def fget(self):
-            return self._crop
-        def fset(self, segment):
-            self._crop = segment
-        def fdel(self):
-            self._crop = None
-        return locals()
-    crop = property(**crop())
+    @property
+    def crop(self):
+        """The crop property."""
+        return self._crop
 
-    def width():
-        doc = "The width property."
-        def fget(self):
-            return self._width
-        def fset(self, value):
-            self._width = value
-        def fdel(self):
-            self._width = 20
-        return locals()
-    width = property(**width())
+    @crop.setter
+    def crop(self, segment: Segment):
+        self._crop = segment
 
-    def __getitem__(self, label):
+    @crop.deleter
+    def crop(self):
+        self._crop = None
+
+    @property
+    def width(self):
+        """The width property"""
+        return self._width
+
+    @width.setter
+    def width(self, value: int):
+        self._width = value
+
+    @width.deleter
+    def width(self):
+        self._width = 20
+
+    def __getitem__(self, label: Label) -> LabelStyle:
         if label not in self._style:
             self._style[label] = next(self._style_generator)
         return self._style[label]
@@ -102,33 +185,33 @@ class Notebook(object):
         ax.axes.get_yaxis().set_visible(yaxis)
         return ax
 
-    def draw_segment(self, ax, segment, y, label=None, boundaries=True):
+    def draw_segment(self, ax, segment: Segment, y, label=None, boundaries=True):
 
         # do nothing if segment is empty
         if not segment:
             return
 
-        linewidth, linestyle, color = self[label]
+        linestyle, linewidth, color = self[label]
 
         # draw segment
         ax.hlines(y, segment.start, segment.end, color,
-                 linewidth=linewidth, linestyle=linestyle, label=label)
+                  linewidth=linewidth, linestyle=linestyle, label=label)
         if boundaries:
             ax.vlines(segment.start, y + 0.05, y - 0.05,
-                      color, linewidth=linewidth, linestyle=linestyle)
+                      color, linewidth=1, linestyle='solid')
             ax.vlines(segment.end, y + 0.05, y - 0.05,
-                      color, linewidth=linewidth, linestyle=linestyle)
+                      color, linewidth=1, linestyle='solid')
 
         if label is None:
             return
 
-    def get_y(self, segments):
+    def get_y(self, segments: Iterable[Segment]) -> np.ndarray:
         """
 
         Parameters
         ----------
-        segments : iterator
-            `Segment` iterator (sorted)
+        segments : Iterable
+            `Segment` iterable (sorted)
 
         Returns
         -------
@@ -167,8 +250,9 @@ class Notebook(object):
 
         return y
 
-
-    def __call__(self, resource, time=True, legend=True):
+    def __call__(self, resource: Resource,
+                 time: bool = True,
+                 legend: bool = True):
 
         if isinstance(resource, Segment):
             self.plot_segment(resource, time=time)
@@ -179,9 +263,8 @@ class Notebook(object):
         elif isinstance(resource, Annotation):
             self.plot_annotation(resource, time=time, legend=legend)
 
-        elif isinstance(resource, Scores):
-            self.plot_scores(resource, time=time, legend=legend)
-
+        elif isinstance(resource, SlidingWindowFeature):
+            self.plot_feature(resource, time=time)
 
     def plot_segment(self, segment, ax=None, time=True):
 
@@ -191,7 +274,7 @@ class Notebook(object):
         ax = self.setup(ax=ax, time=time)
         self.draw_segment(ax, segment, 0.5)
 
-    def plot_timeline(self, timeline, ax=None, time=True):
+    def plot_timeline(self, timeline: Timeline, ax=None, time=True):
 
         if not self.crop and timeline:
             self.crop = timeline.extent()
@@ -200,16 +283,15 @@ class Notebook(object):
 
         ax = self.setup(ax=ax, time=time)
 
-        for segment, y in six.moves.zip(cropped, self.get_y(cropped)):
+        for segment, y in zip(cropped, self.get_y(cropped)):
             self.draw_segment(ax, segment, y)
 
         # ax.set_aspect(3. / self.crop.duration)
 
-
-    def plot_annotation(self, annotation, ax=None, time=True, legend=True):
+    def plot_annotation(self, annotation: Annotation, ax=None, time=True, legend=True):
 
         if not self.crop:
-            self.crop = annotation.get_timeline().extent()
+            self.crop = annotation.get_timeline(copy=False).extent()
 
         cropped = annotation.crop(self.crop, mode='intersection')
         labels = cropped.labels()
@@ -217,58 +299,57 @@ class Notebook(object):
 
         ax = self.setup(ax=ax, time=time)
 
-        for (segment, track, label), y in six.moves.zip(
-                cropped.itertracks(label=True), self.get_y(segments)):
+        for (segment, track, label), y in zip(
+                cropped.itertracks(yield_label=True),
+                self.get_y(segments)):
             self.draw_segment(ax, segment, y, label=label)
 
-        # ax.set_aspect(3. / self.crop.duration)
-
         if legend:
+            H, L = ax.get_legend_handles_labels()
+
+            # corner case when no segment is visible
+            if not H:
+                return
+
             # this gets exactly one legend handle and one legend label per label
             # (avoids repeated legends for repeated tracks with same label)
-            H, L = ax.get_legend_handles_labels()
             HL = groupby(sorted(zip(H, L), key=lambda h_l: h_l[1]),
                          key=lambda h_l: h_l[1])
             H, L = zip(*list((next(h_l)[0], l) for l, h_l in HL))
             ax.legend(H, L, bbox_to_anchor=(0, 1), loc=3,
                       ncol=5, borderaxespad=0., frameon=False)
 
-    def plot_scores(self, scores, ax=None, time=True, legend=True):
+    def plot_feature(self, feature: SlidingWindowFeature,
+                     ax=None, time=True, ylim=None):
 
         if not self.crop:
-            self.crop = scores.to_annotation().get_timeline().extent()
+            self.crop = feature.getExtent()
 
-        cropped = scores.crop(notebook.crop, mode='loose')
-        labels = cropped.labels()
+        window = feature.sliding_window
+        n, dimension = feature.data.shape
+        (start, stop), = window.crop(self.crop, mode='loose',
+                                     return_ranges=True)
+        xlim = (window[start].middle, window[stop].middle)
 
-        data = scores.dataframe_.values
-        m = np.nanmin(data)
-        M = np.nanmax(data)
-        ylim = (m - 0.1 * (M - m), M + 0.1 * (M - m))
+        start = max(0, start)
+        stop = min(stop, n)
+        t = window[0].middle + window.step * np.arange(start, stop)
+        data = feature[start: stop]
 
-        ax = self.setup(ax=ax, yaxis=True, ylim=ylim, time=time)
+        if ylim is None:
+            m = np.nanmin(data)
+            M = np.nanmax(data)
+            ylim = (m - 0.1 * (M - m), M + 0.1 * (M - m))
 
-        for segment, track, label, value in cropped.itervalues():
-            y = value
-            self.draw_segment(ax, segment, y, label=label, boundaries=False)
-
-        # ax.set_aspect(6. / ((ylim[1] - ylim[0]) * self.crop.duration))
-
-        if legend:
-            # this gets exactly one legend handle and one legend label per label
-            # (avoids repeated legends for repeated tracks with same label)
-            H, L = ax.get_legend_handles_labels()
-            HL = groupby(sorted(zip(H, L), key=lambda h_l: h_l[1]),
-                         key=lambda h_l: h_l[1])
-            H, L = zip(*list((next(h_l)[0], l) for l, h_l in HL))
-            ax.legend(H, L, bbox_to_anchor=(0, 1), loc=3,
-                      ncol=5, borderaxespad=0., frameon=False)
+        ax = self.setup(ax=ax, yaxis=False, ylim=ylim, time=time)
+        ax.plot(t, data)
+        ax.set_xlim(xlim)
 
 
 notebook = Notebook()
 
 
-def repr_segment(segment):
+def repr_segment(segment: Segment):
     """Get `png` data for `segment`"""
     import matplotlib.pyplot as plt
     figsize = plt.rcParams['figure.figsize']
@@ -281,7 +362,7 @@ def repr_segment(segment):
     return data
 
 
-def repr_timeline(timeline):
+def repr_timeline(timeline: Timeline):
     """Get `png` data for `timeline`"""
     import matplotlib.pyplot as plt
     figsize = plt.rcParams['figure.figsize']
@@ -294,7 +375,7 @@ def repr_timeline(timeline):
     return data
 
 
-def repr_annotation(annotation):
+def repr_annotation(annotation: Annotation):
     """Get `png` data for `annotation`"""
     import matplotlib.pyplot as plt
     figsize = plt.rcParams['figure.figsize']
@@ -307,120 +388,14 @@ def repr_annotation(annotation):
     return data
 
 
-def repr_scores(scores):
-    """Get `png` data for `scores`"""
+def repr_feature(feature: SlidingWindowFeature):
+    """Get `png` data for `feature`"""
     import matplotlib.pyplot as plt
     figsize = plt.rcParams['figure.figsize']
     plt.rcParams['figure.figsize'] = (notebook.width, 2)
     fig, ax = plt.subplots()
-    notebook.plot_scores(scores, ax=ax)
+    notebook.plot_feature(feature, ax=ax)
     data = print_figure(fig, 'png')
     plt.close(fig)
     plt.rcParams['figure.figsize'] = figsize
     return data
-
-
-def _shorten_long_text(text, max_length=30):
-    suffix = "..."
-    if len(text) > max_length:
-        return text[:max_length - len(suffix)] + suffix
-    else:
-        return text
-
-
-def _clean_text(text):
-
-    only_ascii = six.u(unidecode(text))
-
-    # remove characters that make resulting SVG invalid
-    mapping = {}
-    mapping[ord(u'&')] = None
-    # mapping[ord(u'"')] = u"'"
-
-    return only_ascii.translate(mapping)
-
-
-def _dottable(transcription):
-    # create new graph with graphviz/dot layout instructions
-    # (e.g. graph orientation, node and edge labels, etc.)
-
-    # create a dumb networkx copy to avoid messing with input transcription
-    dottable = transcription.copy()
-
-    # graphviz/dot will display graph from left to right
-    dottable.graph['graph'] = {'rankdir': 'LR'}
-
-    # set shape, label and tooltip of each node
-    for n in transcription.nodes_iter():
-
-        dottable.node[n] = {
-            'label': str(n), 'tooltip': str(n),
-            'shape': 'circle' if n.drifting else 'box'
-        }
-        # 'URL': 'javascript:console.log("{t}")'.format(t=n.T),
-
-    # set edge label
-
-    label_header = (
-        "<<table border='0' cellborder='0' cellspacing='0' cellpadding='3'>"
-    )
-    label_pattern = (
-        "<tr>"
-        "<td align='left'><b>{name}</b></td>"
-        "<td align='left'>{value}</td>"
-        "</tr>"
-    )
-    label_footer = "</table>>"
-    tooltip_pattern = "[{name}] {value}"
-
-    for source, target, key, data in transcription.edges_iter(
-        keys=True, data=True
-    ):
-        tooltip = ""
-        label = ""
-        if data:
-
-            # initialize label table
-            label = label_header
-
-            for name, value in six.iteritems(data):
-                # remove non-ascii characters
-                name = _clean_text(name)
-                value = _clean_text(str(value))
-                # shorten long value
-                short_value = _shorten_long_text(value)
-                # update label and tooltip
-                label += label_pattern.format(name=name, value=short_value)
-                tooltip += tooltip_pattern.format(name=name, value=value)
-
-            # close label table
-            label += label_footer
-
-        if not tooltip:
-            tooltip = " "
-
-        if not label:
-            label = " "
-
-        dottable[source][target][key] = {
-            'label': label,
-            'labeltooltip': tooltip,
-            'edgetooltip': tooltip,
-            'headtooltip': tooltip,
-            'tailtooltip': tooltip,
-        }
-
-    return dottable
-
-
-def _write_temporary_dot_file(transcription):
-    _, path = tempfile.mkstemp('.dot')
-    nx.write_dot(_dottable(transcription), path)
-    return path
-
-
-def repr_transcription(transcription):
-    """Get `svg` data for `transcription`"""
-    path = _write_temporary_dot_file(transcription)
-    data = subprocess.check_output(["dot", "-T", "svg", path]).decode('ascii')
-    return data[data.find("<svg"):]
